-- cbump
local cbump = require "cbump"
local world = cbump.newWorld() -- default is 64

-- camera
local camera = Sprite.new()
local bg = Sprite.new()
local fg = Sprite.new()
camera:setScale(1)
camera:addChild(bg)
camera:addChild(fg)
stage:addChild(camera)

-- usage
--local map = TiledMap.new("tiled/test.lua", world, bg, true, 255, 0, 0, 255)
local map = CBumpTiled.new("tiled/test.lua", world, bg, true)

--function Sprite_Maker:init(xname, xspritesheet, xcols, xrows, xposx, xposy)
local player1 = Sprite_Maker.new("player1", "gfx/player/HQ_Trooper_all.png", 6, 13, 128, 128)
player1:setScale(1)
player1:setAlpha(0.85)
player1.lives = 3
player1.filter = function(item, other)
	if other.name == "coin01" then
		return "cross"
	elseif other.name == "bullet01" then
		return "cross"
	elseif other.name == "exit" then
		return "cross"
	else
		return "slide"
	end
end
fg:addChild(player1)
world:add(player1, player1.x, player1.y, player1.w * player1:getScaleX(), player1.h * player1:getScaleY())

-- GAME LOOP
function onEnterFrame(e)
	-- camera follows player
	local playerposx, playerposy = player1:getPosition()
	playerposx, playerposy = playerposx * camera:getScale(), playerposy * camera:getScale()
	local offsetX = -(playerposx - myappwidth / 2)
	local offsetY = -(playerposy - myappheight / 2)
	camera:setPosition(offsetX, offsetY)

	-- player1
	if player1.lives > 0 then
		updatePlayer1(player1, e.deltaTime)
	else
		scenemanager:changeScene("gameover", 3, transitions[6], easing.outBack)
	end
end

-- FUNCTIONS
function updatePlayer1(xsprite, dt)
	-- player1 pushed by gravity
	xsprite.vy += GRAVITY * dt

	-- cbump
	local goalx = xsprite.x + xsprite.vx * dt
	local goaly = xsprite.y + xsprite.vy * dt
	local nextx, nexty, collisions, len = world:move(xsprite, goalx, goaly, xsprite.filter)
	-- collisions
	for i = 1, len do
		local col = collisions[i]
		-- collectibles collisions
		if col.other.name == "coin01" then
			local sound = coinsnd:play()
			sound:setVolume(0.1)
			col.other.lives -= 1
		end
		-- enemy collision
		if col.other.name == "enemy01" or col.other.name == "enemy02" then
			if col.normal.y == -1 then
				col.other.lives -= 1
				col.item.isattacking = true
			end
		end
		-- sprite collides top or bottom
		if col.normal.y == 1 or col.normal.y == -1 then
			if col.other.name == "coin01" or col.other.name == "exit" then
				-- nothing here
			else
				col.item.vy = 0
			end
		end
		-- sprite is on floor
		if col.normal.y == -1 then
			if col.other.name == "coin01" then
				col.item.isonfloor = false
			elseif col.item.isattacking then
				col.item.vy = -col.item.jumpspeed / 1.5
				col.item.isonfloor = false
				col.item.isattacking = false
			else
				col.item.isonfloor = true
			end
		end
		-- EXIT
		if col.other.name == "exit" then
			if #coins == 0 then
				g_level += 1
				if g_level == 4 then
					scenemanager:changeScene("youwin", 5, transitions[21], easing.outBack)
					g_level = 1
				end
				scenemanager:changeScene("levelx", 2, transitions[2], easing.outBack)
			end
		end
	end

	-- anim state
	if xsprite.isonfloor then
		if xsprite.isshooting then
--			xsprite.animspeed = 1 / 32
			xsprite.currentanim = "shoot"
		else
			if xsprite.vx == 0 then
--				xsprite.animspeed = 1 / 2
				xsprite.currentanim = "idle"
			elseif xsprite.vx ~= 0 then
--				xsprite.animspeed = 1 / 12
				xsprite.currentanim = "walk"
			end
		end
	else
		if xsprite.vy < 0 then
			xsprite.currentanim = "jump_up"
		elseif xsprite.vy >= 0 then
			xsprite.currentanim = "jump_down"
		end
	end

	-- keyboard handling
	if xsprite.iskeyright then
		xsprite.vx += xsprite.accel * dt
		if xsprite.vx > xsprite.maxspeed then xsprite.vx = xsprite.maxspeed end
		xsprite.flip = 1
	elseif xsprite.iskeyleft then
		xsprite.vx -= xsprite.accel * dt
		if xsprite.vx < -xsprite.maxspeed then xsprite.vx = -xsprite.maxspeed end
		xsprite.flip = -1
	else
		xsprite.vx = 0
	end
	if xsprite.iskeyup then
		if xsprite.isonfloor then
			xsprite.vy = -xsprite.jumpspeed
			xsprite.isonfloor = false
		end
	end
	if xsprite.iskeyspace then
	end

	-- anim loop
	if xsprite.currentanim ~= "" then
		xsprite.animtimer = xsprite.animtimer - dt
		if xsprite.animtimer <= 0 then
			xsprite.frame += 1
			xsprite.animtimer = xsprite.animspeed
			if xsprite.frame > #xsprite.anims[xsprite.currentanim] then
				xsprite.frame = 1
				xsprite.isshooting = false
			end
			xsprite.bmp:setTextureRegion(xsprite.anims[xsprite.currentanim][xsprite.frame])
		end
	end

	-- move & flip
	xsprite.x = nextx
	xsprite.y = nexty
	xsprite:setPosition(xsprite.x, xsprite.y)
	xsprite.bmp:setScale(xsprite.flip, 1)
end

stage:addEventListener(Event.ENTER_FRAME, onEnterFrame)
